'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var initialise = require('./initialise.js');
var store = require('./store.js');
var message = require('./message.js');

const define = (name, configFn) => {
  if (customElements.get(name)) return

  customElements.define(
    name,
    class extends HTMLElement {
      async connectedCallback() {
        let nextTickSubscribers = [];
        const config = configFn(this);

        const api = {
          nextTick: (fn) => nextTickSubscribers.push(fn),
        };

        const message$1 = message.Message({
          postPublish: () => {
            nextTickSubscribers.forEach((fn) => fn(state));
            nextTickSubscribers = [];
          },
        });

        const observed = new Set();
        const host = this;

        const wrap = (state) => {
          return new Proxy(state, {
            get(_, name) {
              if (observed.has(name) === false) {
                Object.defineProperty(host, name, {
                  get() {
                    return getState()[name]
                  },
                  set(value) {
                    merge({ [name]: value });
                  },
                });

                observed.add(name);
              }
              return Reflect.get(...arguments)
            },
          })
        };

        const onChangeCallback = (state) => {
          message$1.publish(wrap(state), config);
        };

        const store$1 = store.Store({
          ...config,
          api,
          onChangeCallback,
        });

        const { getState, merge } = store$1;

        const initialState = initialise.initialise(
          this,
          message$1.subscribe,
          config,
          store$1,
          ((state) => (typeof state === "function" ? state({}) : state))(
            config.state || {}
          )
        );

        store$1.set(initialState);
        onChangeCallback(store$1.getState());

        const sa = this.setAttribute;
        this.setAttribute = (name, value) => {
          if (observed.has(name)) {
            merge({ [name]: value });
          }
          return sa.apply(this, [name, value])
        };
        const ra = this.removeAttribute;
        this.removeAttribute = (name) => {
          if (observed.has(name)) {
            merge({ [name]: null });
          }
          return ra.apply(this, [name])
        };

        config.connectedCallback?.(this, store$1);
      }
    }
  );
};

exports.define = define;
