'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var list = require('./list.js');
var xo = require('./xo.js');

function initialise(rootNode, subscribe, config, store, state = {}) {
  const event = {};

  const { elements = [] } = config;

  // derive initial state from input directives...
  elements
    .filter(({ input }) => input)
    .forEach(({ select, input }) => {
      const targets = [...rootNode.querySelectorAll(select)];
      targets.forEach((target) => {
        state = { ...state, [input]: target.value };

        event.input = event.input || [];
        event.input.push({
          select,
          callback: (event) => {
            const { target } = event;
            store.merge({ [input]: target.value });
          },
        });
      });
    });

  // find event listeners
  elements.forEach((c) => {
    const { select, on } = c;
    if (on) {
      Object.entries(on).forEach(([type, callback]) => {
        event[type] = event[type] || [];
        event[type].push({
          select,
          callback,
        });
      });
    }
  });

  //derive initial state from lists...
  elements
    .filter(({ list }) => list)
    .forEach(({ select, list: list$1 }) => {
      const targets = [...rootNode.querySelectorAll(select)];
      targets.forEach((target) => {
        const items = list.listItems(target, list$1.select);
        const curr = list.listData(items);
        state[list$1.from] = curr;
      });
    });

  // delegate from the root node
  Object.entries(event).forEach(([type, listeners]) => {
    rootNode.addEventListener(type, (e) => {
      listeners
        .filter(({ select }) => e.target.matches(select))
        .forEach(({ select, callback }) => {
          if (typeof callback === "function") {
            callback(e, store);
          }
          if (typeof callback === "string") {
            const { target } = e;

            const targets = [...rootNode.querySelectorAll(select)];

            const index = targets.indexOf(target);

            store.dispatch(callback, { event: e, index });
          }
        });
    });
  });

  const onChange = (state) => {
    // lists first
    elements
      .filter(({ list }) => list)
      .forEach(({ select, list: list$1 }) => {
        const targets = [...rootNode.querySelectorAll(select)];
        targets.forEach((target) => {
          const items = list.listItems(target, list$1.select);
          const curr = list.listData(items);
          const next = state[list$1.from];
          list.listSync(target, items, curr, next, list$1.template);
        });
      });

    // then the rest...
    elements.forEach((c) => {
      const { select, attribute, input } = c;

      const targets = [...rootNode.querySelectorAll(select)];

      targets.forEach((target, i) => {
        if (attribute) {
          xo.write(target, attribute(state, i));
        }
        if (input) {
          const value = state[input];

          if (target.value !== value) target.value = value;
        }
      });
    });
  };

  subscribe(onChange);

  return state
}

exports.initialise = initialise;
